/******************************************************************************/
// Dungeon Keeper 2 EngineSurface classes
/******************************************************************************/
/** @file engine_surfaces.h
 *     EngineSurface classes implementation.
 * @par Purpose:
 *     Header file. Defines exported routines from engine_surfaces.cpp
 * @par Comment:
 *     None.
 * @author   Tomasz Lis
 * @date     01 Oct 2009 - 05 Oct 2009
 * @par  Copying and copyrights:
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 */
/******************************************************************************/

#ifndef ENGN_SURF_H
#define ENGN_SURF_H

/******************************************************************************/
class GraphicsParams {
public:
  void set_color_params(unsigned int mask_r, unsigned int mask_g, unsigned int mask_b, unsigned int mask_a, int bpp);
  unsigned long field_0;
  unsigned long bpp1;
  unsigned long bytes_per_pix;
  unsigned long bitspp_g;
  unsigned long bitspp_b;
  unsigned long bitspp_a;
  unsigned long bitspp_r;
  unsigned long mask_g1;
  unsigned long mask_b1;
  unsigned long mask_a1;
  unsigned long mask_r1;
  unsigned char field_2C;
  unsigned long field_2D;
  unsigned long field_31;
  unsigned long field_35;
  unsigned long field_39;
  unsigned long bpp3;
  unsigned long field_41;
  unsigned long field_45;
  unsigned long field_49;
  unsigned long field_4D;
  unsigned long bpp4;
  unsigned long mask_g2;
  unsigned long mask_b2;
  unsigned long mask_a2;
  unsigned long mask_r2;
};


class CEngineSurfaceBase {
  public:
    CEngineSurfaceBase(long w, long h);
    virtual ~CEngineSurfaceBase();
    virtual void fill(unsigned short val);
    virtual bool put_image(class CEngineSurfaceBase *dst, int pos_x, int pos_y);
    virtual bool get_image(class CEngineSurfaceBase *src, int pos_x, int pos_y);
    virtual int func_10(void) = 0;
    virtual bool out_supported(void) = 0;
    virtual unsigned char *get_out_buffer(void);
    virtual void func_1C(unsigned char *buf);
    virtual unsigned char *get_data_buffer(long *len);

    long width;
    long height;
    unsigned long line_len;
    class GraphicsParams *graph_params;
};


class CEngineCompressedSurface : public CEngineSurfaceBase {
  public:
    CEngineCompressedSurface(long w, long h, long len);
    virtual ~CEngineCompressedSurface();
    virtual bool put_image(class CEngineSurfaceBase *dst, int pos_x, int pos_y);
    virtual bool get_image(class CEngineSurfaceBase *src, int pos_x, int pos_y);
    virtual int func_10(void);
    virtual bool out_supported(void);
    virtual unsigned char *get_data_buffer(long *len);

    unsigned char *data;
    long data_len;
};

/*
class CEngineDDSurface : public CEngineSurfaceBase {
  // TODO
};
*/

/******************************************************************************/
extern class GraphicsParams *graphics_params;
/******************************************************************************/
#endif
